#pragma once
#include <fstream>
#include "Exception.h"
#include <vector>
#include <algorithm>
#include <iterator>
#include <typeinfo>

void createData(const std::string&);
void createDataContainer(const std::string&);

template <class Container>
class Data {
private:
	Container cont;

	using Type = Container::value_type;

public:

	Data() : cont() {}

	Data(int size) : cont(size) {}

	Container& loadData(const std::string& filepath) { //!!! Лишнее копирование при возврате (исправлено)
		std::ifstream file(filepath);
		if (!file)
			throw FileException(1);
		Type a;
		while (!file.eof()) {
			file >> a;
			cont.push_back(a);
		}
		return cont;
	}

	Container& streamloadData(const std::string& filepath) {//!!! Лишнее копирование при возврате (исправлено)
		std::ifstream file(filepath);
		std::copy(std::istream_iterator<Type>(file), std::istream_iterator<Type>(), std::back_inserter(cont));
		return cont;
	}

	void modify() {
		int count = 0;
		Type S = Type();

		for (auto i = cont.begin(); i != cont.end(); ++i)
			S += *i;

		Type m = S / cont.size();
		for (auto i = cont.begin(); i != cont.end(); ++i) {
			if (*i > 0) {
				++count;
				if (count == 3) {
					count = 0;
					*i = m;
				}
			}
		}
	}

	static void modify_iterator(Container::iterator a, Container::iterator b) {
		int count = 0;
		Type S = Type();

		for (auto i = a; i != b; ++i) {
			S += *i;
			++count;
		}

		Type m = S / count;
		count = 0;
		for (auto i = a; i != b; ++i) {
			if (*i > 0) {
				++count;
				if (count == 3) {
					count = 0;
					*i = m;
				}
			}
		}
	}

	void modify_iterator() {
		Data::modify_iterator(cont.begin(), cont.end());
	}

	void modify_for_each() {
		Type S = Type(); 

		for (auto i = cont.begin(); i != cont.end(); ++i)
			S += *i;

		Type m = S / cont.size();

		static int count = 0;

		std::for_each(cont.begin(), cont.end(),
			[&m, &count](Type& x) {              //!!! Захват m желательно делать по ссылке (исправлено)
				 //!!! Лучше завести внешнюю переменную и захватить ее по сссылке (исправлено)
				if (x > 0) {
					++count;
					if (count == 3) {
						count = 0;
						x = m;
					}
				}
			}
		);
	}

	void outputResult(const std::string& filepath) {
		std::ofstream file(filepath);
		if (!file)
			throw FileException(1);
		for (auto i = cont.begin(); i != cont.end(); ++i)
			file << *i << " ";
	}

	void outputResultContainer(const std::string& filepath) {
		std::ofstream file(filepath);
		if (!file)
			throw FileException(1);
		std::copy(cont.begin(), cont.end(), std::ostream_iterator<Type>(file, " "));
	}
};